<?php

namespace Drupal\email_tfa\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\BareHtmlPageRendererInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\user\Form\UserLoginForm;
use Drupal\user\UserAuthInterface;
use Drupal\user\UserFloodControlInterface;
use Drupal\user\UserStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Email tfa form.
 */
class EmailTfaLoginForm extends UserLoginForm {

  /**
   * The user flood control service.
   *
   * @var \Drupal\user\UserFloodControl
   */
  protected $userFloodControl;

  /**
   * The user storage.
   *
   * @var \Drupal\user\UserStorageInterface
   */
  protected $userStorage;

  /**
   * The user authentication object.
   *
   * @var \Drupal\user\UserAuthInterface
   */
  protected $userAuth;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The bare HTML renderer.
   *
   * @var \Drupal\Core\Render\BareHtmlPageRendererInterface
   */
  protected $bareHtmlPageRenderer;

  /**
   * The route match object for the current page.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;


  /**
   * The aggregator.settings config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * Constructs a new subscriber after login.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\user\UserFloodControlInterface $user_flood_control
   *   The user flood control service.
   * @param \Drupal\user\UserStorageInterface $user_storage
   *   The user storage.
   * @param \Drupal\user\UserAuthInterface $user_auth
   *   The user authentication object.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Drupal\Core\Render\BareHtmlPageRendererInterface $bare_html_renderer
   *   The bare HTML renderer.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The factory for the temp store object.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              UserFloodControlInterface $user_flood_control,
                              UserStorageInterface $user_storage,
                              UserAuthInterface $user_auth,
                              RendererInterface $renderer,
                              BareHtmlPageRendererInterface $bare_html_renderer,
                              PrivateTempStoreFactory $temp_store_factory,
  ) {
    parent::__construct($user_flood_control, $user_storage, $user_auth, $renderer, $bare_html_renderer);
    $this->config = $config_factory->get('email_tfa.settings');
    $this->tempStoreFactory = $temp_store_factory->get('email_tfa');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('user.flood_control'),
      $container->get('entity_type.manager')->getStorage('user'),
      $container->get('user.auth'),
      $container->get('renderer'),
      $container->get('bare_html_page_renderer'),
      $container->get('tempstore.private'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'email_tfa_email_tfa_login';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $uid = NULL, $hash = NULL) {
    $form = parent::buildForm($form, $form_state);
    $config = \Drupal::config('email_tfa.settings');
    $status = $config->get('status');
    if ($this->getRequest()->query->get('destination')) {
      // save the destination in the $form_state to be used on submit.
      $form_state->set('destination', $this->getRequest()->query->get('destination'));
      // remove the destination from the query string.
      $this->getRequest()->query->remove('destination');
    }
    $form['#cache'] = ['max-age' => 0];
    if (!$status) {
      return $form;
    }
    $form['#submit'][] = '::emailTfaLoginFormRedirect';
    $form['#cache'] = ['max-age' => 0];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function emailTfaLoginFormRedirect(array &$form, FormStateInterface $form_state) {
    $uid = $form_state->get('uid');
    $user = $this->userStorage->load($uid);
    $hash = _email_tfa_hash($uid, $user->getAccountName(), $user->getPassword());
    delete_temp_store($hash);
    $this->tempStoreFactory->set($hash . '_email_tfa_expire_time', time());
    $this->tempStoreFactory->set($hash . '_email_tfa_resend_mail', TRUE);
    if ($form_state->get('destination')) {
      $form_state->setRedirect('email_tfa.email_tfa_verify_login',
        [
          'uid' => $form_state->get('uid'),
          'hash' => $hash,
          'destination' => $form_state->get('destination'),
        ]
      );
    }
    else {
      $form_state->setRedirect('email_tfa.email_tfa_verify_login',
        [
          'uid' => $form_state->get('uid'),
          'hash' => $hash,
        ]
      );
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = \Drupal::config('email_tfa.settings');
    $status = $config->get('status');
    if (!$status) {
      $form = parent::submitForm($form, $form_state);
      return $form;
    }
  }

}
