<?php

namespace Drupal\email_tfa\Form;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a Email tfa form.
 */
class EmailTfaVerifyLoginForm extends FormBase {

  use StringTranslationTrait;

  /**
   * The aggregator.settings config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;


  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new subscriber after login.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The factory for the temp store object.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, PrivateTempStoreFactory $temp_store_factory, MessengerInterface $messenger, MailManagerInterface $mail_manager, LanguageManagerInterface $language_manager, EntityTypeManagerInterface $entity_type_manager) {
    $this->config = $config_factory->get('email_tfa.settings');
    $this->tempStoreFactory = $temp_store_factory->get('email_tfa');
    $this->messenger = $messenger;
    $this->mailManager = $mail_manager;
    $this->languageManager = $language_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('tempstore.private'),
      $container->get('messenger'),
      $container->get('plugin.manager.mail'),
      $container->get('language_manager'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'email_tfa_email_tfa_verify_login';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $uid = NULL, $hash = NULL) {
    $form['#cache']['max-age'] = 0;
    if ($this->getRequest()->query->get('destination')) {
      // save the destination in the $form_state to be used on submit.
      $form_state->set('destination', $this->getRequest()->query->get('destination'));
      // remove the destination from the query string.
      $this->getRequest()->query->remove('destination');
    }
    $timeouts = $this->config->get('timeouts');
    if (!empty($this->tempStoreFactory->get($hash . '_email_tfa_expire_time'))) {
      if (time() - (int) $this->tempStoreFactory->get($hash . '_email_tfa_expire_time') > $timeouts) {
        delete_temp_store($hash);
        $url = Url::fromRoute('user.page')->toString();
        $response = new RedirectResponse($url);
        $response->send();
        $this->messenger->addError($this->t('Two-factor Authentication is Expired.'));
        exit;
      }
    }

    $form_state->set('result', FALSE);
    $user = $this->entityTypeManager->getStorage('user')->load($uid);
    $tracks = $this->config->get('tracks');
    $user_one = $this->config->get('user_one');
    $ignore_role = $this->config->get('ignore_role');

    // If user one is exclude and its the current one.
    if ($user_one == 1 && $uid == 1 && $tracks == 'globally_enabled') {
      $form_state->set('result', FALSE);
    }
    // Globally enabled pathway && some roles if selected will excluded.
    elseif ($tracks == 'globally_enabled' && !_email_tfa_in_array_any($user->getRoles(), $ignore_role)) {
      // Check if user is NOT already verified.
      $form_state->set('result', TRUE);
    }
    elseif ($tracks == 'optionally_by_users') {
      if (!empty(_email_tfa_user_current_status()) && _email_tfa_user_current_status() == 1) {
        $form_state->set('result', TRUE);
      }
    }

    if (!$form_state->get('result')) {
      user_login_finalize($user);
      // redirect to user page
      $url = Url::fromRoute('entity.user.canonical', ['user' => $user->id()])->toString();
      $response = new RedirectResponse($url);
      $response->send();
    }

    if (!$this->tempStoreFactory->get($hash . '_email_tfa_send_mail')) {
      $this->sendEmail($uid, $hash);
    }

    // Fill the form with the user data.
    $form_state->set('uid', $uid);
    $form_state->set('hash', $hash);
    $form['verify'] = [
      '#type' => 'textfield',
      '#title' => $this->config->get('security_code_label_text'),
      '#description' => $this->config->get('security_code_description_text'),
      '#size' => 60,
      '#required' => TRUE,
    ];

    // @todo add submit button for test mode if test mode is enabled.
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->config->get('security_code_verify_text'),
      '#button_type' => 'primary',
      '#submit' => ['::loginSubmitForm'],
      '#validate' => ['::loginValidateForm'],
    ];

    if ($this->tempStoreFactory->get($hash . '_email_tfa_resend_mail')) {
      $form['resend'] = [
        '#type' => 'submit',
        '#value' => $this->config->get('security_code_resend_text'),
        '#name' => 'resend',
        '#button_type' => 'primary',
        // Skip validation.
        '#limit_validation_errors' => [],
        '#submit' => ['::sendEmailSubmitForm'],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function loginValidateForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getTriggeringElement()['#name'] != 'resend') {
      $hash = $form_state->get('hash');
      if ($form_state->getValue('verify') != $this->tempStoreFactory->get($hash . '_email_tfa_code')) {
        $form_state->setErrorByName('verify', $this->config->get('verification_failed_message'));
      }

      $uid = $form_state->get('uid');
      $user = $this->entityTypeManager->getStorage('user')->load($uid);
      if (_email_tfa_hash($uid, $user->getAccountName(), $user->getPassword()) != $hash) {
        $form_state->setErrorByName('verify', $this->config->get('not_authorized_message'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function loginSubmitForm(array &$form, FormStateInterface $form_state) {
    $hash = $form_state->get('hash');
    delete_temp_store($hash);
    $uid = $form_state->get('uid');
    $user = $this->entityTypeManager->getStorage('user')->load($uid);
    $this->messenger->addStatus($this->config->get('verification_succeeded_message'));
    if (empty($form_state->get('destination'))) {
      $form_state->setRedirect(
        'entity.user.canonical',
        ['user' => $user->id()]
      );
    }
    else {
      $this->getRequest()->query->set('destination', $form_state->get('destination'));
    }
    user_login_finalize($user);
  }

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account, $uid = NULL, $hash = NULL) {
    $user = $this->entityTypeManager->getStorage('user')->load($uid);
    if (_email_tfa_hash($uid, $user->getAccountName(), $user->getPassword()) != $hash) {
      return AccessResult::forbidden();
    }

    if (empty($this->tempStoreFactory->get($hash . '_email_tfa_expire_time'))) {
      return AccessResult::forbidden();
    }

    return AccessResult::allowed();
  }

  /**
   * Generates a random number with a configured length.
   *
   * @return int
   *   A random number with the configured length.
   */
  protected function generateCode() {
    $length = $this->config->get('security_code_length');
    // Cast result of pow to int because it can return float.
    $min = (int) pow(10, $length - 1);
    $max = (int) pow(10, $length) - 1;
    return mt_rand($min, $max);
  }

  /**
   * Resend email.
   */
  public function sendEmailSubmitForm(array &$form, FormStateInterface $form_state) {
    $hash = $form_state->get('hash');
    $uid = $form_state->get('uid');
    if ($this->tempStoreFactory->get($hash . '_email_tfa_resend_mail')) {
      $this->sendEmail($uid, $hash);
      $this->tempStoreFactory->set($hash . '_email_tfa_resend_mail', FALSE);
    }
    else {
      $this->messenger->addStatus($this->t('The email has already been sent successfully.'));
    }
    if ($form_state->get('destination')) {
      $form_state->setRedirect(
        'email_tfa.email_tfa_verify_login',
        [
          'uid' => $form_state->get('uid'),
          'hash' => $form_state->get('hash'),
          'destination' => $form_state->get('destination'),
        ]
      );
    }
  }

  /**
   * Sends an email to the user.
   */
  protected function sendEmail($uid, $hash) {
    // @todo block send email for 5 minutes if send 5 times.
    $user = $this->entityTypeManager->getStorage('user')->load($uid);
    $code = $this->generateCode();
    if ($this->config->get('dev_mode')) {
      $this->messenger->addStatus('Note: Test mode is enabled. The code is: ' . $code . '.');
    }
    $this->tempStoreFactory->set($hash . '_email_tfa_send_mail', TRUE);
    $this->tempStoreFactory->set($hash . '_email_tfa_code', $code);
    $this->mailManager->mail('email_tfa', 'send_email_tfa', $user->getEmail(), $this->languageManager->getDefaultLanguage()
      ->getId(), [
        'user' => $user,
        'email_tfa' => $this->tempStoreFactory->get($hash . '_email_tfa_code'),
      ]);
    $this->messenger->addStatus('Email sent successfully.');

  }

}
